// motorzxuno.h

// Los bits del color de paleta van en forma GGGRRRBB
// Los pixels en cada byte van invertidos, el segundo es el primero (al ser los bits de menos peso)
// Es decir 22221111 es el formato de los bits de pixel en cada byte


// Funciones:

// ___________________________________________
// Borra la pantalla a un color
// ___________________________________________

void cls (int color)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld a, (hl)
		rla			;rotamos 4 bits a la izquierda
		rla
		rla
		rla			;ya tenemos el color del primer pixel
		add a, (hl)		;aadimos el color del segundo pixel
		
		ld hl, 16384
		ld de, 16385
		ld bc, 6143
		ld (hl), a
		ldir
	#endasm
}


// ___________________________________________
// Delay, produce una pausa en el programa
// ___________________________________________

void delay (int delay)
{
	#asm
	ld hl,2
	add hl,sp
	ld c, 0
	ld b, 2
	uno:
	ld e, (hl)	;operando base
	ld d, 0
	dos:
	dec de
	ld a,d
	or e
	jp nz,dos
	dec bc
	ld a,b
	or c
	jp nz,uno
	ret
	#endasm
}

// ___________________________________________
// Funcin para sacar un dato por un puerto
// ___________________________________________

void port_out (int port, int value)
{
	#asm
	ld hl,2
	add hl,sp
	ld a, (hl)
	inc hl
	inc hl
	ld c, (hl)
	inc hl
	ld b, (hl)
	out (c),a
	#endasm
}

// ___________________________________________
// Definir paleta de color
// ___________________________________________
void escribe_paleta (int indice, int color)
{
	// Los bits del color van en forma GGGRRRBB
	port_out (48955, indice);
	port_out (65339, color);
}


// ___________________________________________
// Funcin para leer un puerto
// ___________________________________________

int __FASTCALL__ port_in (int port)
{
	#asm
        	ld      b, h
        	ld      c, l
        	ld      h, 0
        	in      l, (c)
	#endasm
}


// ___________________________________________
// Espera interrupcin y para la CPU
// ___________________________________________

void wait_int (void)
{
	#asm
		halt
	#endasm
}

// ___________________________________________
// Dibuja un pixel a color
// ___________________________________________

void plot (unsigned char color, unsigned int x, unsigned int y)
{
	#asm
		ld hl,2		; Pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)	; D es el valor Y
		inc hl
		inc hl
		ld e, (hl)	; E es el valor X
		rr e		; quitamos el valor de las unidades (los pixels van de dos en dos por byte)
		ld a,0
		jr c, next	; si es impar no hacemos nada (se dibuja por defecto a la derecha)
		ld a,1
		next:
		inc hl
		inc hl
		ld l, (hl)	; L es el color
		ld h,a

		ld a, d		; recuperamos el valor vertical
		rrca
	    rrca		; rotamos para dejar su valor en mltiplos de 64 (linea, de dos en dos pixels)
		and 192		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d		; cargamos el valor vertical
		rrca
		rrca		; rotamos para quedarnos con los bits altos
		and 63		; borramos el resto de bits
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla

		ld a,(de)
		rr h
		jr c, next2
		and 240
		or l
		jr fin
		next2:
		and 15
		rl l
		rl l
		rl l
		rl l
		or l
		fin:
		ld (de), a		
	#endasm
}

// ___________________________________________
// Posiciona un Sprite de 8x8 a color
// ___________________________________________

void put_sprite_x8 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	// -------------------------------------------
	// RUTINA DE IMPRESION DE UN SPRITE 8x8 PIXELS
	// EN CUALQUIER POSICION DE BYTES (cada byte dos pixels horizontales)
	// ENTRADAS:
	// D ser la posicin del cursor vertical en pixels
	// E ser la posicin del cursor horizontal en parejas de pixels
	// HL es la posicin de memoria donde tenemos el sprite
	// SALIDAS: se escribe en el mapa de pantalla
	// ADVERTENCIAS: no comprueba lmites de pantalla
	// -------------------------------------------
	#asm
		ld hl,2		; Pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)	; D es el valor Y
		inc hl
		inc hl
		ld e, (hl)	; E es el valor X
		inc hl
		inc hl
		ld a, (hl)
	    	inc hl
	    	ld h, (hl)
	    	ld l, a		; HL es el puntero al sprite
	
		ld a, d		; recuperamos el valor vertical
		rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 64 (linea, de dos en dos pixels)
		and 192		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d		; cargamos el valor vertical
		rrca
		rrca		; rotamos para quedarnos con los bits altos
		and 63		; borramos el resto de bits
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
	
		ld b,8		; Indicamos que vamos a pintar 8 lneas
		draw:		; dibujamos 8 pixels (4 bytes)
		ld c,4		; Indicamos que vamos a pintar 4 pares de pixels
		ldi
		ldi
		ldi
		ldi
		ld a,e
		add a,60	; incrementamos una lnea y retornamos los pixels dibujados
		ld e,a
		jr nc, sigue
		inc d		; incrementamos D si sale acarrero al incrementar E en 64 bytes
		sigue:
		djnz draw
	
		ret
	
	#endasm
}

// ___________________________________________
// Posiciona un Sprite de 10x10 a color
// ___________________________________________

void put_sprite_x10 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	// -------------------------------------------
	// RUTINA DE IMPRESION DE UN SPRITE 10x10 PIXELS
	// EN CUALQUIER POSICION DE BYTES (cada byte dos pixels horizontales)
	// ENTRADAS:
	// D ser la posicin del cursor vertical en pixels
	// E ser la posicin del cursor horizontal en parejas de pixels
	// HL es la posicin de memoria donde tenemos el sprite
	// SALIDAS: se escribe en el mapa de pantalla
	// ADVERTENCIAS: no comprueba lmites de pantalla
	// -------------------------------------------
	#asm
		ld hl,2		; Pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)	; D es el valor Y
		inc hl
		inc hl
		ld e, (hl)	; E es el valor X
		inc hl
		inc hl
		ld a, (hl)
	    	inc hl
	    	ld h, (hl)
	    	ld l, a		; HL es el puntero al sprite
	
		ld a, d		; recuperamos el valor vertical
		rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 64 (linea, de dos en dos pixels)
		and 192		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d		; cargamos el valor vertical
		rrca
		rrca		; rotamos para quedarnos con los bits altos
		and 63		; borramos el resto de bits
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
	
		ld b,10		; Indicamos que vamos a pintar 10 lneas
		draw3:		; dibujamos 10 pixels (5 bytes)
		ld c,5		; Indicamos que vamos a pintar 5 pares de pixels
		ldi
		ldi
		ldi
		ldi
		ldi
		ld a,e
		add a,59	; incrementamos una lnea y retornamos los pixels dibujados
		ld e,a
		jr nc, sigue3
		inc d		; incrementamos D si sale acarrero al incrementar E en 64 bytes
		sigue3:
		djnz draw3
	
		ret
	
	#endasm
}

// ___________________________________________
// Posiciona un Sprite de 8x8 a color con mscara
// ___________________________________________

void put_msprite_x8 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	// -------------------------------------------
	// RUTINA DE IMPRESION DE UN SPRITE 8x8 PIXELS CON MSCARA (color 0 paleta)
	// EN CUALQUIER POSICION DE BYTES (cada byte dos pixels horizontales)
	// ENTRADAS:
	// D ser la posicin del cursor vertical en pixels
	// E ser la posicin del cursor horizontal en parejas de pixels
	// HL es la posicin de memoria donde tenemos el sprite
	// SALIDAS: se escribe en el mapa de pantalla
	// ADVERTENCIAS: no comprueba lmites de pantalla
	// -------------------------------------------
	unsigned int xx;
	unsigned int yy=8;
	unsigned char *pantalla=16384+x+y*64;
	unsigned int buffer;
	unsigned int dibuja;
	unsigned int a,b;

	while (yy>0)
	{
		xx=4;
		while (xx>0)
		{
			buffer=*posicion;
			a = (buffer&15);
			b = (buffer&240);
			dibuja = 10*(a>0)+(b>0);
			switch(dibuja)
			{
				case 0: dibuja = *pantalla; break;
				case 1: dibuja = b+((*pantalla)&15); break;
				case 10: dibuja = a+((*pantalla)&240); break;
				case 11: dibuja = buffer; break;
			}
			*pantalla=dibuja;
			posicion++;
			pantalla++;
			xx--;
		}
		yy--;
		pantalla=pantalla+60;
	}
}

// ___________________________________________
// Posiciona un Sprite de 10x10 a color con mscara
// ___________________________________________

void put_msprite_x10 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	// -------------------------------------------
	// RUTINA DE IMPRESION DE UN SPRITE 10x10 PIXELS CON MSCARA (color 0 paleta)
	// EN CUALQUIER POSICION DE BYTES (cada byte dos pixels horizontales)
	// ENTRADAS:
	// D ser la posicin del cursor vertical en pixels
	// E ser la posicin del cursor horizontal en parejas de pixels
	// HL es la posicin de memoria donde tenemos el sprite
	// SALIDAS: se escribe en el mapa de pantalla
	// ADVERTENCIAS: no comprueba lmites de pantalla
	// -------------------------------------------
	unsigned int xx;
	unsigned int yy=10;
	unsigned char *pantalla=16384+x+y*64;
	unsigned int buffer;
	unsigned int dibuja;
	unsigned int a,b;

	while (yy>0)
	{
		xx=5;
		while (xx>0)
		{
			buffer=*posicion;
			a = (buffer&15);
			b = (buffer&240);
			dibuja = 10*(a>0)+(b>0);
			switch(dibuja)
			{
				case 0: dibuja = *pantalla; break;
				case 1: dibuja = b+((*pantalla)&15); break;
				case 10: dibuja = a+((*pantalla)&240); break;
				case 11: dibuja = buffer; break;
			}
			*pantalla=dibuja;
			posicion++;
			pantalla++;
			xx--;
		}
		yy--;
		pantalla=pantalla+59;
	}
}

